import {AfterViewInit, ChangeDetectorRef, Component, OnInit, ViewChild} from '@angular/core';
import {ActivatedRoute, ParamMap, Router} from '@angular/router';
import {SpeciesAnalysis} from 'src/app/models/SpeciesAnalysis';
import {DataService} from 'src/app/services/data.service';
import {PaginatedDataProvider} from '../../models/pagination/paginated-data-provider';
import {ListingOptions} from '../../models/pagination/listing-options';
import {Observable, of} from 'rxjs';
import {PageData} from '../../models/pagination/page-data';
import {SpeciesRow} from '../../models/SpeciesRow';
import {MatPaginatedDataSource} from '../../models/pagination/mat-paginated-data-source';
import {MatSort} from '@angular/material/sort';
import {environment} from '../../../environments/environment';
import {MatPaginator} from "@angular/material/paginator";


class SpeciesPaginatedProvider implements PaginatedDataProvider<SpeciesRow> {

  private checkedAnalysis: Map<string, Map<string, boolean>>;

  public constructor(
    private readonly family: string,
    private readonly species: string,
    private readonly dataService: DataService
  ) {
    this.checkedAnalysis = new Map<string, Map<string, boolean>>();
  }

  public getCurrentSpeciesName(): string | undefined {
    return this.species;
  }


  public isChecked(analysis: SpeciesRow): boolean {
    return this.checkedAnalysis.get(analysis.regionName)?.get(analysis.modelName) === true;
  }

  public onToggleRow(analysis: SpeciesRow): void {
    const checked = this.isChecked(analysis);
    // @ts-ignore
    this.checkedAnalysis.get(analysis.regionName).set(analysis.modelName, !checked);
  }

  public setCheckedAnalysis(): void {
    this.dataService.getSpecieResults(this.family, this.species).subscribe(rowDataArray => {
        Object.values(rowDataArray).forEach(rowData => {
          if (!this.checkedAnalysis?.has(rowData.regionName)) {
            this.checkedAnalysis?.set(rowData.regionName, new Map<string, boolean>());
          }
          // @ts-ignore
          this.checkedAnalysis.get(rowData.regionName).set(rowData.modelName, rowData.hasPredictedAnalysis());
        });
      }
    );
  }


  public list(options: ListingOptions): Observable<PageData<SpeciesRow>> {
    this.setCheckedAnalysis();
    if (this.species === undefined) {
      return of(PageData.EMPTY_PAGE);
    } else {
      return this.dataService.listSpeciesResults(this.family, this.species, options);
    }
  }
}

@Component({
  selector: 'slf-result',
  templateUrl: './result.component.html',
  styleUrls: ['./result.component.scss']
})
export class ResultComponent implements OnInit, AfterViewInit {
  @ViewChild(MatPaginator) public paginator?: MatPaginator;
  @ViewChild(MatSort) public sort?: MatSort;
  public id: string;
  public readonly familyName: string;
  private readonly speciesName: string;
  public speciesAnalysis?: SpeciesAnalysis;
  public speciesAnalysisNames: string[];
  public rowData: SpeciesRow[];

  private _dataSource?: MatPaginatedDataSource<SpeciesRow>;
  private dataProvider?: SpeciesPaginatedProvider;

  public readonly displayedColumns: string[] =
    ['predicted', 'region', 'model', 'min_tree_distance', 'fbox', 'isoelectric_point'];

  constructor(
    private readonly route: ActivatedRoute,
    private readonly router: Router,
    private readonly changeDetector: ChangeDetectorRef,
    private readonly dataService: DataService
  ) {
    const paramMap = this.route.snapshot.paramMap;
    if (!(paramMap.has('family') && paramMap.has('id'))) {
      this.router.navigate(['/']);
      throw new Error('Missing familyName and id');
    }
    this.familyName = <string>paramMap.get('family');
    this.id = <string>paramMap.get('id');
    this.speciesName = this.id.substring(0, this.id.indexOf('_' + this.familyName));
    this.speciesAnalysisNames = [];
    this.rowData = [];
  }

  public ngOnInit(): void {
    this.dataService.getFamily(this.familyName)
      .subscribe(familyAnalysis => {
        this.speciesAnalysis = familyAnalysis.speciesAnalysis[this.id];
        this.speciesAnalysisNames = Object.keys(familyAnalysis.speciesAnalysis)
          .filter(speciesName => speciesName.includes(this.speciesName));
      });

    this.changeId(this.id);
  }

  public ngAfterViewInit(): void {
    this._dataSource?.setControls(this.paginator, this.sort);
  }

  private changeId(id: string): void {
    this.id = id;
    const paramMap: ParamMap = this.route.snapshot.paramMap;
    if (this.id !== paramMap.get('id')) {
      this.router.navigate(['species', this.familyName, this.id])
        .then(() => this.updateDataSource());
    } else {
      this.updateDataSource();
    }
  }

  private updateDataSource(): void {
    this.dataProvider = new SpeciesPaginatedProvider(this.familyName, this.id, this.dataService);
    this._dataSource = new MatPaginatedDataSource<SpeciesRow>(this.dataProvider);

    if (this.paginator !== undefined && this.sort !== undefined) {
      this.paginator.firstPage();
      this._dataSource?.setControls(this.paginator, this.sort);
    }
  }

  public get dataSource(): MatPaginatedDataSource<SpeciesRow> {
    if (this._dataSource === undefined) {
      throw new Error('dataProvider is undefined');
    }
    return this._dataSource;
  }

  public onChangeResults(value: string): void {
    this.changeId(value);
  }

  public isChecked(analysis: SpeciesRow): boolean {
    return <boolean>this.dataProvider?.isChecked(analysis);
  }

  public onToggleRow(analysis: SpeciesRow): void {
    this.dataProvider?.onToggleRow(analysis);
    this.changeDetector.detectChanges();
  }

  public buildFileUrl(filePath: string): string {
    return `${environment.filesUrl}/${filePath}`;
  }
}
